package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import ru.dbotthepony.mc.otm.*;
import ru.dbotthepony.mc.otm.block.entity.worker.BlockEntityMatteryWorker;
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJob;
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJobStatus;
import ru.dbotthepony.mc.otm.capability.*;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.capability.IMatterGridCell;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.matter.MatterRegistry;
import ru.dbotthepony.mc.otm.menu.MatterDecomposerMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;

public class BlockEntityMatterDecomposer extends BlockEntityMatteryWorker implements IMatterGridCell {
	private static final TranslatableComponent MACHINE_NAME = new TranslatableComponent("block.overdrive_that_matters.matter_decomposer");
	private boolean valid = true;
	public final MatterHandlerCapability matter = new MatterHandlerCapability(this::setChanged, IMatterHandler.MatterDirection.EXTRACT, new BigDecimal("20"));
	private final LazyOptional<IMatterHandler> matter_resolver = LazyOptional.of(() -> matter);

	// вход, выход
	public final MatteryContainer item_container = new MatteryContainer(this::setChanged, 2);

	private final LazyOptional<IItemHandler> handler_resolver = LazyOptional.of(() -> item_container.handler(
		(slot, stack) -> slot == 0 && MatterRegistry.hasMatterValue(stack),
		(slot, amount, stack) -> slot == 1
	));

	public BlockEntityMatterDecomposer(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.MATTER_DECOMPOSER, p_155229_, p_155230_);
		energy = new MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, new BigDecimal(400_000), new BigDecimal(2000), new BigDecimal(2000));
	}

	@Override
	protected Component getDefaultDisplayName() {
		return MACHINE_NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new MatterDecomposerMenu(containerID, inventory, this);
	}

	@Nullable
	@Override
	public IPatternStorage getPatternStorage() {
		return null;
	}

	@Override
	public CompoundTag save(CompoundTag nbt) {
		nbt.put("work_slots", item_container.serializeNBT());
		nbt.put("matter_capability", matter.serializeNBT());

		return super.save(nbt);
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);

		if (nbt.contains("matter_capability") && nbt.get("matter_capability") instanceof CompoundTag tag)
			matter.deserializeNBT(tag);

		item_container.deserializeNBT(nbt.get("work_slots"));
	}

	@Override
	public void reviveCaps() {
		valid = true;
		super.reviveCaps();
	}

	@Override
	public void invalidateCaps() {
		valid = false;
		super.invalidateCaps();
	}

	private final LazyOptional<IMatterGridCell> resolver_grid = LazyOptional.of(() -> this);

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid) {
			if (cap == MatteryCapability.MATTER)
				return matter_resolver.cast();

			if (cap == MatteryCapability.MATTER_CELL)
				return resolver_grid.cast();

			if (cap == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
				return handler_resolver.cast();
		}

		return super.getCapability(cap, side);
	}

	@Nonnull
	@Override
	protected MachineJobStatus onJobFinish(MachineJob job) {
		BigDecimal matter_value = MatterRegistry.getMatterValue(job.stack());
		matter.receiveMatterInner(matter_value, false);
		return new MachineJobStatus();
	}

	@Nullable
	@Override
	protected MachineJob getNextJob() {
		ItemStack stack = item_container.getItem(0);

		if (!stack.isEmpty()) {
			ItemStack copy = stack.copy();
			copy.setCount(1);

			if (MatterRegistry.canDecompose(copy)) {
				BigDecimal matter_value = MatterRegistry.getMatterValue(copy);

				if (!matter_value.equals(BigDecimal.ZERO) && matter.canReceiveAll(matter_value)) {
					stack.shrink(1);
					return new MachineJob(copy, matter_value.doubleValue() * 12_500d);
				}
			}
		}

		return null;
	}

	public static <T extends BlockEntity> void tick(Level level, BlockPos blockPos, BlockState blockState, T t) {
		if (t instanceof BlockEntityMatterDecomposer tile) {
			tile.batteryChargeLoop();
			tile.workerLoop();

			if (tile.matter.getStoredMatter().compareTo(BigDecimal.ZERO) > 0 && tile.grid != null) {
				BigDecimal diff = tile.matter.extractMatterInner(tile.matter.getStoredMatter(), true);
				BigDecimal diff2 = tile.grid.softPushMatter(diff, true);

				tile.matter.extractMatterInner(diff2, false);
				tile.grid.softPushMatter(diff2, false);
			}
		}
	}

	private MatterGrid grid;

	@Override
	public void setRemoved() {
		if (level != null && !level.isClientSide && grid != null)
			grid.untrack(this);

		super.setRemoved();
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (!p_155231_.isClientSide)
			MatterGrid.scheduleDiscoverNeighbours(this, getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Nullable
	@Override
	public IMatterHandler getMatterHandler() {
		return valid ? matter : null;
	}

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}

	private static final BigDecimal BASE_CONSUMPTION = new BigDecimal(240);

	@Override
	protected BigDecimal getBaseConsumption() {
		return BASE_CONSUMPTION;
	}
}
