package ru.dbotthepony.mc.otm.menu.widget;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.GameRenderer;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.client.render.RenderHelper;
import ru.dbotthepony.mc.otm.client.screen.UVWindingOrder;

abstract public class GaugeWidget extends AbstractWidget {
	public enum GaugeDirection {
		TOP_TO_BOTTOM,
		BOTTOM_TO_TOP,
		LEFT_TO_RIGHT,
		RIGHT_TO_LEFT
	}

	public GaugeWidget(MatteryMenu menu, int x, int y) {
		super(menu, x, y);
	}

	public GaugeWidget(MatteryMenu menu) {
		super(menu);
	}

	public GaugeDirection fill_order = GaugeDirection.BOTTOM_TO_TOP;

	abstract public float getLevel();

	protected void setupGaugeDrawing() {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, RenderHelper.WIDGETS);
	}

	@Override
	public void renderBackground(PoseStack pose, float local_x, float local_y) {
		setupGaugeDrawing();

		var order = getUVWindwing();

		RenderHelper.drawTexturedRectAuto(pose, local_x + x, local_y + y, getImageWidth(), getImageHeight(), getImageX(), getImageY(), order);
		float level = getLevel();

		if (level > 0.01f) {
			if (fill_order == GaugeDirection.BOTTOM_TO_TOP) {
				int intHeight = (int) Math.floor(level * (float) getImageHeight() + 0.5f);

				RenderHelper.drawTexturedRectAuto(
					pose,
					local_x + x,
					local_y + y + getImageHeight() - intHeight,
					getImageWidth(),
					intHeight,
					getImageX() + getImageWidth(),
					getImageY() + getImageHeight() - intHeight,
					order);

			} else if (fill_order == GaugeDirection.TOP_TO_BOTTOM) {
				RenderHelper.drawTexturedRectAuto(
					pose,
					local_x + x,
					local_y + y,
					getImageWidth(),
					(float) Math.floor(level * (float) getImageHeight() + 0.5f),
					getImageX() + getImageWidth(),
					getImageY(),
					order);

			} else if (fill_order == GaugeDirection.LEFT_TO_RIGHT) {
				// forward order
				if (order.u0 == UVWindingOrder.U0) {
					RenderHelper.drawTexturedRectAuto(
						pose,
						local_x + x,
						local_y + y,
						(float) Math.floor(level * (float) getImageWidth() + 0.5f),
						getImageHeight(),
						getImageX() + getImageWidth(),
						getImageY(),
						order);
				} else {
					// backward order
					var value = (float) Math.floor(level * (float) getImageWidth() + 0.5f);

					RenderHelper.drawTexturedRectAuto(
						pose,
						local_x + x + getImageWidth() - value,
						local_y + y,
						value,
						getImageHeight(),
						getImageX() + getImageWidth(),
						getImageY(),
						order);
				}
			} else if (fill_order == GaugeDirection.RIGHT_TO_LEFT) {
				// forward order
				if (order.u0 == UVWindingOrder.U0) {
					var value = (float) Math.floor(level * (float) getImageWidth() + 0.5f);

					RenderHelper.drawTexturedRectAuto(
						pose,
						local_x + x + getImageWidth() - value,
						local_y + y,
						value,
						getImageHeight(),
						getImageX() + getImageWidth(),
						getImageY(),
						order);
				} else {
					// backward order
					RenderHelper.drawTexturedRectAuto(
						pose,
						local_x + x,
						local_y + y,
						(float) Math.floor(level * (float) getImageWidth() + 0.5f),
						getImageHeight(),
						getImageX() + getImageWidth(),
						getImageY(),
						order);
				}
			}
		}
	}
}
